@@ -4,8 +4,8 @@ sub new {
     my($self,@arg) = @_;
     bless [@arg], $self;
 }
-sub con { shift->[0] }
-sub cou { shift->[1] }
+sub continent { shift->[0] }
+sub country { shift->[1] }
 sub url { shift->[2] }
 
 package CPAN::FirstTime;
@@ -41,6 +41,10 @@ sub init {
     
     my($ans,$default,$local,$cont,$url,$expected_size);
     
+    #
+    # Files, directories
+    #
+
     print qq{
 
 The CPAN module needs a directory of its own to cache important
@@ -66,7 +70,10 @@ First of all, I\'d like to create this directory. Where?
     }
 
     $default = $cpan_home;
-    $ans = prompt("CPAN build and cache directory?",$default);
+    until (-d ($ans = prompt("CPAN build and cache directory?",$default)) && -w _) {
+	print "Couldn't find directory $ans
+  or directory is not writable. Please retry.\n";
+    }
     File::Path::mkpath($ans); # dies if it can't
     $CPAN::Config->{cpan_home} = $ans;
     
@@ -82,6 +89,10 @@ next question.
     $CPAN::Config->{keep_source_where} = MM->catdir($CPAN::Config->{cpan_home},"sources");
     $CPAN::Config->{build_dir} = MM->catdir($CPAN::Config->{cpan_home},"build");
 
+    #
+    # Cache size, Index expire
+    #
+
     print qq{
 
 How big should the disk cache be for keeping the build directories
@@ -96,6 +107,10 @@ with all the intermediate files?
     # XXX This the time when we refetch the index files (in days)
     $CPAN::Config->{'index_expire'} = 1;
 
+    #
+    # External programs
+    #
+
     print qq{
 
 The CPAN module will need a few external programs to work
@@ -105,7 +120,7 @@ properly. Please correct me, if I guess the wrong path for a program.
 
     my(@path) = split($Config{path_sep},$ENV{PATH});
     my $prog;
-    for $prog (qw/gzip tar unzip make/){
+    for $prog (qw/gzip tar unzip make lynx ftp/){
 	my $path = $CPAN::Config->{$prog} || find_exe($prog,[@path]) || $prog;
 	$ans = prompt("Where is your $prog program?",$path) || $path;
 	$CPAN::Config->{$prog} = $ans;
@@ -115,9 +130,16 @@ properly. Please correct me, if I guess the wrong path for a program.
 	    find_exe("more",[@path]) || "more";
     $ans = prompt("What is your favorite pager program?",$path) || $path;
     $CPAN::Config->{'pager'} = $ans;
+    $path = $CPAN::Config->{'shell'} || $ENV{SHELL} || "";
+    $ans = prompt("What is your favorite shell?",$path) || $path;
+
+    #
+    # Arguments to make etc.
+    #
+
     print qq{
 
-Every Makefile.PL is run by perl in a seperate process. Likewise we
+Every Makefile.PL is run by perl in a separate process. Likewise we
 run \'make\' and \'make install\' in processes. If you have any parameters
 \(e.g. PREFIX, INSTALLPRIVLIB, UNINST or the like\) you want to pass to
 the calls, please specify them here.
@@ -134,6 +156,30 @@ the calls, please specify them here.
     $CPAN::Config->{make_install_arg} =
 	prompt("Parameters for the 'make install' command?",$default);
 
+    #
+    # Alarm period
+    #
+
+    print qq{
+
+Sometimes you may wish to leave the processes run by CPAN alone
+without caring about them. As sometimes the Makefile.PL contains
+question you\'re expected to answer, you can set a timer that will
+kill a 'perl Makefile.PL' process after the specified time in seconds.
+
+If you set this value to 0, these processes will wait forever.
+
+};
+
+    $default = $CPAN::Config->{inactivity_timeout} || 0;
+    $CPAN::Config->{inactivity_timeout} =
+	prompt("Timout for inacivity during Makefile.PL?",$default);
+
+
+    #
+    # MIRRORED.BY
+    #
+
     $local = 'MIRRORED.BY';
     if (@{$CPAN::Config->{urllist}||[]}) {
 	print qq{
@@ -141,8 +187,8 @@ I found a list of URLs in CPAN::Config and will use this.
 You can change it later with the 'o conf' command.
 
 }
-    } elsif (-f $local) { # if they really have a MIRRORED.BY in the
-                     # current directory, we can't help
+    } elsif (-f $local) { # if they really have a wrong MIRRORED.BY in
+                          # the current directory, we can't help
 	read_mirrored_by($local);
     } else {
 	$CPAN::Config->{urllist} ||= [];
@@ -156,6 +202,19 @@ Please enter it here: };
 	}
     }
 
+    print qq{
+
+If you\'re accessing the net via proxies, you can specify them in the
+CPAN configuration or via environment variables. The variable in
+the \$CPAN::Config takes precedence.
+
+};
+
+    for (qw/ftp_proxy http_proxy no_proxy/) {
+	$default = $CPAN::Config->{$_} || $ENV{$_};
+	$CPAN::Config->{$_} = prompt("Your $_?",$default);
+    }
+
     # We don't ask that now, it will be noticed in time....
     $CPAN::Config->{'inhibit_startup_message'} = 0;
 
@@ -259,7 +318,7 @@ file:, ftp: or http: URL, or "q" to finish selecting.
 	my $sel;
 	if ($ans =~ /^\d/) {
 	    my $this = $valid[$ans-1];
-	    my($con,$cou,$url) = ($this->con,$this->cou,$this->url);
+	    my($con,$cou,$url) = ($this->continent,$this->country,$this->url);
 	    push @{$CPAN::Config->{urllist}}, $url unless $seen{$url}++;
 	    delete $all{$con}{$cou}{$url};
 	    #	    print "Was a number [$ans] con[$con] cou[$cou] url[$url]\n";
